//拓展Format函数以获取时间
Date.prototype.Format = function(fmt) { //author: meizz 
    var o = {
        "M+": this.getMonth() + 1, //月份 
        "d+": this.getDate(), //日 
        "h+": this.getHours(), //小时 
        "m+": this.getMinutes(), //分 
        "s+": this.getSeconds(), //秒 
        "S": this.getMilliseconds() //毫秒 
    };
    if (/(y+)/.test(fmt)) fmt = fmt.replace(RegExp.$1, (this.getFullYear() + "").substr(4 - RegExp.$1.length));
    for (var k in o)
        if (new RegExp("(" + k + ")").test(fmt)) fmt = fmt.replace(RegExp.$1, (RegExp.$1.length == 1) ? (o[k]) : (("00" + o[k]).substr(("" + o[k]).length)));
    return fmt;
}

// 云函数入口文件
const cloud = require('wx-server-sdk')

cloud.init()
const db = cloud.database()

//评论
var comment = async (event) => {
	var res = await db.collection("sM_productInfo").doc(event.productId).update({
		data:{
			comment: db.command.push({
			  name:event.userInfos.userName,
				avatar:event.userInfos.avatarUrl,
				userId:event.userInfos._id,
				time: (new Date(new Date().getTime() + 1000 * 60 * 60 * 8)).Format("yyyy-MM-dd hh:mm:ss"),
				msg: event.msg,
			})
		}
	}).then(res => {
		if(res.stats.updated !== 1) 
			return {
				errCode:301,
				errMsg:'评论添加失败(id错误)',
				id:event.productId
			}; 
		else return {
			errCode: 200,
			errMsg: 'ok',
		}
	}).catch(err => {
		return {
			errCode: 302,
			errMsg: '评论添加失败',
			err: err,
		};
	})
	return res;
}

//回复
var reply = async(event) => {	
	//取出原有的comment加以修改
	var oldInfo = await db.collection("sM_productInfo").doc(event.productId).get();
	if(!oldInfo.data.comment[event.replyNum].reply) oldInfo.data.comment[event.replyNum].reply = [];
	oldInfo.data.comment[event.replyNum].reply.push({
		name:event.userInfos.userName,
		avatar: event.userInfos.avatarUrl,
		userId: event.userInfos._id,
		msg: event.msg,
		time: (new Date(new Date().getTime() + 1000 * 60 * 60 * 8)).Format("yyyy-MM-dd hh:mm:ss"),
	})
	//放回去
	var res = await db.collection("sM_productInfo").doc(event.productId).update({
		data:{
			comment: db.command.set(oldInfo.data.comment)
		}
	}).then(res => {
		if(res.stats.updated !== 1) 
			return {
				errCode:303,
				errMsg:'回复添加失败(id错误)',
				id:event.productId
			}; 
		else return {
			errCode: 200,
			errMsg: 'ok',
		}
	}).catch(err => {
		return {
			errCode: 304,
			errMsg: '回复添加失败',
			err: err,
		};
	})

	return res;
}

//通知
var notice = async (event) => {
	//commonMsg
	var res = await db.collection('sM_userInfo').doc(event.replyId).update({
		data: {
			unreadMsg: db.command.push({
				productId:event.productId,
				productName:event.productName,
				time: (new Date(new Date().getTime() + 1000 * 60 * 60 * 8)).Format("yyyy-MM-dd hh:mm:ss"),
				iscomment:event.iscomment,
				isWant:false,
			})
		}
	}).then(res => {
		if(res.stats.updated !== 1) 
			return {
				errCode:305,
				errMsg:'通知添加失败(id错误)',
				id:event.replyId
			}; 
		else return {
			errCode: 200,
			errMsg: 'ok',
		}
	}).catch(err => {
		return {
			errCode: 306,
			errMsg: '通知添加失败',
			err: err,
		};
	})
	return res;
}

// 云函数入口函数
exports.main = async (event, context) => {
	if (event.iscomment) {
		var res = await comment(event);
	} else {
		var res = await reply(event);
	}
	if(res.errCode !== 200) return res;
	noticeRes = await notice(event);
	return noticeRes;
}

/*
	api说明：
		参数：
			iscomment: true 是评论
				userInfos: 服务器返回的用户信息
				productId: 商品id
				msg: 评论内容
				productName: 商品名
			iscomment: false 是回复
				userInfos: 同上
				productid: 同上
				msg: 评论内容
				replyNum: 回复的第几个评论
				replyId: 被回复的人的id
				productName: 商品名
		备注：回复的时候，直接在评论下边累加，方便前端for遍历，虽然看着有点难受，可能在后续版本改一下
		作者：RainSun
		时间：2019/05/01
*/

/*
	错误报告：
	注：err表示catch函数返回的错误，msg就是问题描述

		正常：
			code:200
			msg:正常的返回值或者'ok'

		评论添加失败(id错误)：
			code:301
			msg:
			id: 
		
		评论添加失败：
			code: 302
			msg:
			err:

		回复添加失败(id错误)：
			code: 303
			msg:
			id:

		回复添加失败：
			code: 304
			msg:
			err:

		通知添加失败(id错误)：
			code:305
			msg:
			id:
		
		通知添加失败:
			code:306
			msg:
			err:

*/