// 云函数入口文件
const cloud = require('wx-server-sdk')

cloud.init()
const db = cloud.database()

//100 时间排序
var timeSort = (status) => {
  //101 正序 102 倒序
  if (status == 101) return db.collection('sM_productInfo').where({issall:false}).orderBy('time', 'asc').get();
  else return db.collection('sM_productInfo').orderBy('time', 'desc').get();
}

//200 浏览量排序
var viewSort = (status) => {
  //201 正序 202 倒序
  if (status == 201) return db.collection('sM_productInfo').where({issall:false}).orderBy('viewNum', 'asc').get();
  else return db.collection('sM_productInfo').orderBy('viewNum', 'desc').get();
}

//300 价格排序
var priceSort = (status) => {
  //301 正序 302 倒序
  if (status == 301) return db.collection('sM_productInfo').where({issall:false}).orderBy('price', 'asc').get();
  else return db.collection('sM_productInfo').orderBy('price', 'desc').get();
}

//400 免费
var freeSort = (status) => {
  //401 时间正序 402 时间倒序
  if (status == 401) return db.collection('sM_productInfo').where({isfree: true,issall:false}).orderBy('time', 'asc').get();
  else return db.collection('sM_productInfo').where({ isfree: true, issall:false}).orderBy('time', 'desc').get();
}

//500 模糊搜索（默认时间倒序）
var fuzzySearch = (argument) => {
  return db.collection('sM_productInfo').where({
    title: db.RegExp({ //使用正则表达式
      regexp: argument,
      options: 'i',   //大小写不区分
    }),
		issall:false
  }).orderBy('time', 'desc').get();
}

//600 标签分类
var tagClassify = async (status) => {
  /*
    目前的标签分为 610:#图书#  620:#电子#  630:#体育# 640:#乐器# 650:#生活#
  */
  var res = null;
  switch (parseInt(status / 10)) {
    case 61:
      // 611 时间正序 612 时间倒序 613 价格正序 614 价格倒序 615 浏览量正序 616 浏览量倒序
      if (status = 611) res = await tagSearch("图书","time","asc");
      else if (status = 612) res = await tagSearch("图书", "time", "desc");
      else if (status = 613) res = await tagSearch("图书", "price", "asc");
      else if (status = 614) res = await tagSearch("图书", "price", "desc");
      else if (status = 615) res = await tagSearch("图书", "viewNum", "asc");
      else if (status = 616) res = await tagSearch("图书","viewNum","desc");
      break;
    case 62:
      // 621 时间正序 622 时间倒序 623 价格正序 624 价格倒序 625 浏览量正序 626 浏览量倒序
      if (status = 621) res = await tagSearch("电子", "time", "asc");
      else if (status = 622) res = await tagSearch("电子", "time", "desc");
      else if (status = 623) res = await tagSearch("电子", "price", "asc");
      else if (status = 624) res = await tagSearch("电子", "price", "desc");
      else if (status = 625) res = await tagSearch("电子", "viewNum", "asc");
      else if (status = 626) res = await tagSearch("电子", "viewNum", "desc");
      break;
    case 63:
      // 631 时间正序 632 时间倒序 633 价格正序 634 价格倒序 635 浏览量正序 636 浏览量倒序
      if (status = 631) res = await tagSearch("体育", "time", "asc");
      else if (status = 632) res = await tagSearch("体育", "time", "desc");
      else if (status = 633) res = await tagSearch("体育", "price", "asc");
      else if (status = 634) res = await tagSearch("体育", "price", "desc");
      else if (status = 635) res = await tagSearch("体育", "viewNum", "asc");
      else if (status = 636) res = await tagSearch("体育", "viewNum", "desc");
      break;
    case 64:
      // 641 时间正序 642 时间倒序 643 价格正序 644 价格倒序 645 浏览量正序 646 浏览量倒序
      if (status = 641) res = await tagSearch("乐器", "time", "asc");
      else if (status = 642) res = await tagSearch("乐器", "time", "desc");
      else if (status = 643) res = await tagSearch("乐器", "price", "asc");
      else if (status = 644) res = await tagSearch("乐器", "price", "desc");
      else if (status = 645) res = await tagSearch("乐器", "viewNum", "asc");
      else if (status = 646) res = await tagSearch("乐器", "viewNum", "desc");
      break;
    case 65:
      // 651 时间正序 652 时间倒序 653 价格正序 654 价格倒序 655 浏览量正序 656 浏览量倒序
      if (status = 651) res = await tagSearch("生活", "time", "asc");
      else if (status = 652) res = await tagSearch("生活", "time", "desc");
      else if (status = 653) res = await tagSearch("生活", "price", "asc");
      else if (status = 654) res = await tagSearch("生活", "price", "desc");
      else if (status = 655) res = await tagSearch("生活", "viewNum", "asc");
      else if (status = 656) res = await tagSearch("生活", "viewNum", "desc");
      break;
    default:
      return false;
  }
  return res;
}

//标签查询
var tagSearch = (tag,type,order) => {
  return db.collection('sM_productInfo').where({
    tag:tag,
		issall:false
  }).orderBy(type, order).get();
}

var getAllData = async () => {
  const MAX_LIMIT = 100
  const countResult = await db.collection('sM_productInfo').count()
  const total = countResult.total
  if(total === 0) return {
    errCode: 601,
    errMsg:'空表'
  }
  // 计算需分几次取
  const batchTimes = Math.ceil(total / 100)
  // 承载所有读操作的 promise 的数组
  const tasks = []
  for (let i = 0; i < batchTimes; i++) {
    const promise = db.collection('sM_productInfo').skip(i * MAX_LIMIT).limit(MAX_LIMIT).get()
    tasks.push(promise)
  }
  // 等待所有
  return (await Promise.all(tasks)).reduce((acc, cur) => {
    return {
      data: acc.data.concat(cur.data),
      errMsg: acc.errMsg,
      errCode:200,
    }
  })
}

// 云函数入口函数
exports.main = async (event, context) => {
  const wxContext = cloud.getWXContext()
  var res;
  switch (parseInt(event.status / 100)) {
    case 1:
      res = await timeSort(event.status);
      break;
    case 2:
      res = await viewSort(event.status);
      break;
    case 3:
      res = await priceSort(event.status);
      break;
    case 4:
      res = await freeSort(event.status);
      break;
    case 5:
      res = await fuzzySearch(event.argument);
      break;
    case 6:
      res = await tagClassify(event.status);
      break;
    case 7:
      res = await getAllData();
      break;
    default:
      res = false;
  }
  return res;
}

/*
	api说明：
    功能：获取商品列表
    参数：
      status:
        101：时间正序
        102：时间倒序
        201：浏览量正序
        202：浏览量倒序
        301：价格正序
        302：价格倒序
        401：免费->时间正序
        402：免费->时间倒序
        500：模糊搜索（默认时间倒序）
        600：标签
          10：#图书#
          20：#电子#
          30：#体育#
          40：#乐器#
          50：#生活#
            1：时间正序
            2：时间倒序
            3：价格正序
            4：价格倒序
            5：浏览量正序
            6：浏览量倒序
      argument：模糊搜索内容      
    备注：
      argument参数只在status为500时使用，为模糊查询的字符串。
      标签搜索例子：611->图书标签下的时间正序
		作者：RainSun
		时间：2019/05/01
*/
/*
  日志：
    20190502：
      也不知道为什么把查询写接口里了，权限设置一下，小程序直接查应该更快，
      无所谓了，反正都写完了，到时候前端移植一下就好
      然后就是，，云函数里默认返回值就100条，这个没控制，
      到时候在前端里边加一个分页控制20个左右断一下，下拉刷新什么的来一套
*/