// 云函数入口文件
const cloud = require('wx-server-sdk')

cloud.init()
const db = cloud.database();

//改商品信息
var rewriteProduct = async(event) => {
	var res = db.collection('sM_productInfo').doc(event.productId).update({
		data: {
			wantUser: db.command.push(event.userInfos._id)
		}
	}).then(res => {
		if(res.stats.updated !== 1) return {
			errCode: 401,
			errMsg: "向商品信息里边添加'想要'userId错误（productId错误）",
			id: event.productId
		}
		return {
			errCode: 200,
			errMsg: 'ok',
		}
	}).catch(err => {
		return {
			errCode: 402,
			errMsg: "向商品信息里边添加'想要'userId错误",
			err: err,
		}
	})
	return res;
}

//改个人信息
var rewriteUser = async(event) => {
	var res = db.collection('sM_userInfo').doc(event.userInfos._id).update({
		data: {
			want: db.command.push({
				productId:event.productId,
				productName:event.productName
			})
		}
	}).then(res => {
		if(res.stats.updated !== 1) return {
			errCode: 403,
			errMsg: "向用户信息添加想要商品错误（userId错误）",
			id: event.userInfo._id
		}
		return {
			errCode: 200,
			errMsg: 'ok',
		}
	}).catch(err => {
		return {
			errCode: 404,
			errMsg: "向用户信息添加想要商品错误",
			err: err,
		}
	})
	return res;
}

//删除商品里边的用户信息
var removeProduct = async(event) => {
	var oldInfo = await db.collection("sM_productInfo").doc(event.productId).get();
	var oldData = oldInfo.data.wantUser;
	//删除用户在商品里边的记录
	var length = oldData.length;
    for (var i = 0; i < length; i++) {
        if (oldData[i] == event.userInfos._id) {
            if (i == 0) {
                oldData.shift(); //删除并返回数组的第一个元素
            }
            else if (i == length - 1) {
                oldData.pop();  //删除并返回数组的最后一个元素
            }
            else {
                oldData.splice(i, 1); //删除下标为i的元素
            }
        }
	}
	//放回去
	var res = db.collection('sM_productInfo').doc(event.productId).update({
		data:{
			wantUser: db.command.set(oldData)
		}
	}).then(res => {
		if(res.stats.updated !== 1) 
			return {
				errCode:405,
				errMsg:'商品信息取消收藏失败(id错误)',
				id:event.productId
			}; 
		else return {
			errCode: 200,
			errMsg: 'ok',
		}
	}).catch(err => {
		return {
			errCode: 406,
			errMsg: '商品信息取消收藏失败',
			err: err,
		};
	})
  	return res;
}

//删除用户里边的商品信息
var removeUser = async (event) =>{
	var oldInfo = await db.collection("sM_userInfo").doc(event.userInfos._id).get();
	var oldData = oldInfo.data.want;
	//删除商品在用户里边的记录
	var length = oldData.length;
    for (var i = 0; i < length; i++) {
        if (oldData[i].productId == event.productId) {
            if (i == 0) {
                oldData.shift(); //删除并返回数组的第一个元素
            }
            else if (i == length - 1) {
                oldData.pop();  //删除并返回数组的最后一个元素
            }
            else {
                oldData.splice(i, 1); //删除下标为i的元素
            }
        }
	}
	//放回去
	var res = db.collection('sM_userInfo').doc(event.userInfos._id).update({
		data: {
			want: db.command.set(oldData)
		}
	}).then(res => {
		if(res.stats.updated !== 1) return {
			errCode: 407,
			errMsg: "用户信息取消收藏失败(id错误)",
			id: event.userInfo._id
		}
		return {
			errCode: 200,
			errMsg: 'ok',
		}
	}).catch(err => {
		return {
			errCode: 408,
			errMsg: "用户信息取消收藏失败",
			err: err,
		}
	})
	return res;
}

// 云函数入口函数
exports.main = async (event, context) => {
	if(event.isWant) {
		var rePRes = await rewriteProduct(event);
		if(rePRes.errCode !== 200) return rePRes;
		var reURes = await rewriteUser(event);
		return reURes;
	} else {
		var rePRes = await removeProduct(event);
		if(rePRes.errCode !== 200) return rePRes;
		var reURes = await removeUser(event);
		return reURes;
	}
}

/*
	api说明：
		功能：用户点击‘收藏’，将用户id加进商品信息，将商品id加进用户信息
		参数：
			isWant:true, 加收藏
				productId:
				userInfos: 云函数返回的用户信息
				productName:
			isWant:flase,取消收藏
				productId:
				userInfos: 云函数返回的用户信息
		作者：RainSun
		时间：2019/05/16
*/
/*
	错误报告：
		注：
			err表示catch函数返回的错误，msg就是问题描述

		正常：
			code:200
			msg:正常的返回值或者'ok'

		向商品信息里边添加'想要'userId错误（productId错误）
			code:401
			msg:
			id:

		向商品信息里边添加'想要'userId错误
			code:402
			msg:
			err:

		向用户信息添加想要商品错误（userId错误）
			code:403
			msg:
			id:

		向用户信息添加想要商品错误
			code:404
			msg:
			err:
		商品信息取消收藏失败(id错误)
			code:405
			msg:
			id:
		商品信息取消收藏失败
			code:406
			msg:
			err:
		用户信息取消收藏失败(id错误)
			code:407
			msg:
			id:
		用户信息取消收藏失败
			code:408
			msg:
			err:
*/	